
/*------------------------------------------------------------------------
    File        : aa-2202-after-01-aa-purch-invoice-sequence.p
    Purpose     :

    Syntax      :

    Description : Conversie programma purchase invoice sequences
                  Converteert sequences per department uit aa-fin_department_cr naar sequences in aa-sequence tabel.

    Author(s)   :
    Created     : Thu Apr 26 10:06:20 CEST 2012
    Notes       :
                  - zet nooit een include in je conversie programma's, deze staan niet in de uitleverset
                    (met uitzondering van system/conversion.conversion_progress.i)
                  - als de database triggers niet persee af hoeven te gaan , disable ze dan

                  - gebruik de procedure 'DisplayProgress' om info/voortgang en tellers te tonen
                  - gebruik de FIELDS optie als je maar weinig velden hoeft te updaten.

                  - OF gebruik het TRANSACTION keyword als je bij de FOR EACH een EXCLUSIVE-LOCK opgeeft
                    zie onderstaand voorbeeld A

                  - OF gebruik een buffer voor de tabel als je de FOR EACH met NO-LOCK leest
                    zie onderstaand voorbeeld B

  ----------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */

/* Deze standaard include toevoegen aan conversie programmatuur,
   o.a de DisplayProgress procedure en input paramatere naar de
   aanroepende procedure worden hier in gedefineerd.
   Hierdoor wordt de aansluiting naar de update procedure gemaakt.

   Opmerking: Deze include wordt opgenomen in de System/comversion
              directory welke als ongecompileerd bestand.
              Het is de enige include die opgenomen mag worden in
              de conversie programmatuur.
*/
/* standalone preprocessor even op iets anders zetten als je deze handmatig wil draaien */
&SCOPED-DEFINE standalone NO

&IF {&standalone} = NO &THEN
{System/conversion/conversion_progress.i}
&ENDIF

/* ********************  Preprocessor Definitions  ******************** */


/* ***************************  Main Block  *************************** */
DEF VAR cProgram                      AS CHAR NO-UNDO.
DEF VAR cAction                       AS CHAR NO-UNDO.
DEF VAR cTable                        AS CHAR NO-UNDO.
DEF VAR cSeqcode                      AS CHAR NO-UNDO.

/* purchase invoice sequences - > in aa-sequence tabel */
cProgram =  THIS-PROCEDURE:FILE-NAME.
cAction  = "Convert purch. invoice sequences per department".
cTable = "aa-fin_department_cr".


/* --------------------------------------------------------------
  Disable triggers op de tabel indien dit mogelijk of
  onderzoek de effecten als de triggers aan moeten
  blijven staan
   -------------------------------------------------------------- */
DISABLE TRIGGERS FOR LOAD OF aa-sequence.

FOR EACH sp-company NO-LOCK:
  FOR EACH aa-fin_Department_cr WHERE aa-fin_department_cr.companycode = sp-company.companycode NO-LOCK:
    cSeqcode = SUBSTITUTE("purchase&10000.&2", sp-company.companycode, aa-fin_department_cr.department_cr_code).

    IF (aa-fin_department_cr.invoicenr_start > 0 AND aa-fin_department_cr.invoicenr_end > 0)
      AND (NOT CAN-FIND(aa-sequence WHERE aa-sequence.sequencecode = cSeqcode NO-LOCK)) THEN DO:

      CREATE aa-sequence.
      ASSIGN aa-sequence.sequencecode = cSeqCode /* per department, per jaar ondersteunen we (nog) niet, codes wel consistent zien te houden. */
             aa-sequence.initial_value = aa-fin_department_cr.invoicenr_start
             aa-sequence.limit = aa-fin_department_cr.invoicenr_end
             aa-sequence.current_value = aa-fin_department_cr.invoicenr_current + 1
             aa-sequence.previous_value = aa-fin_department_cr.invoicenr_current
             aa-sequence.cycle_at_limit = YES /* vorige implementatie ging ook rond, dus dat houden we intact */
             aa-sequence.interval = 1
             aa-sequence.des = SUBSTITUTE("company &1, cred. dept. &2",sp-company.companycode,aa-fin_department_cr.department_cr_code).
             .
     END.
     &IF {&standalone} = NO &THEN
     giDone = giDone + 1.
     IF giDone MODULO giModulo = 0 THEN RUN DisplayProgress (cProgram, cAction, cTable    , giTotal     , giDone).
     &ENDIF
  END.
END.
&IF {&standalone} = NO &THEN
RUN DisplayProgress (cProgram, cAction, cTable    , giTotal     , giDone).
&ENDIF

/* invoice sequences - > in sp-sequence tabel */
&IF {&standalone} = NO &THEN
cAction  = "Convert invoice sequences per department".
cTable = "aa-fin_department_db".
giDone = 0.
&ENDIF

/* --------------------------------------------------------------
  Disable triggers op de tabel indien dit mogelijk of
  onderzoek de effecten als de triggers aan moeten
  blijven staan
   -------------------------------------------------------------- */
DISABLE TRIGGERS FOR LOAD OF sp-sequence.

FOR EACH sp-company NO-LOCK:
  FOR EACH aa-fin_Department_db WHERE aa-fin_department_db.companycode = sp-company.companycode NO-LOCK:
    cSeqcode = SUBSTITUTE("&1invoicenr.&2", sp-company.companycode, aa-fin_department_db.department_db_code).

    IF  (aa-fin_department_db.invoicenr_start > 0 AND aa-fin_department_db.invoicenr_end > 0)
      AND (NOT CAN-FIND(sp-sequence WHERE sp-sequence.sequencecode = cSeqcode NO-LOCK)) THEN DO:

      CREATE sp-sequence.
      ASSIGN sp-sequence.sequencecode = cSeqCode /* per department, per jaar ondersteunen we (nog) niet, codes wel consistent zien te houden. */
             sp-sequence.initial_value = aa-fin_department_db.invoicenr_start
             sp-sequence.limit = aa-fin_department_db.invoicenr_end
             sp-sequence.current_value = aa-fin_department_db.invoicenr_current + 1
             sp-sequence.previous_value = aa-fin_department_db.invoicenr_current
             sp-sequence.cycle_at_limit = YES /* vorige implementatie ging ook rond, dus dat houden we intact */
             sp-sequence.interval = 1
             sp-sequence.des = SUBSTITUTE("company &1, debt. dept. &2",sp-company.companycode,aa-fin_department_db.department_db_code).
             .
     END.
     &IF {&standalone} = NO &THEN
     giDone = giDone + 1.
     IF giDone MODULO giModulo = 0 THEN RUN DisplayProgress (cProgram, cAction, cTable    , giTotal     , giDone).
     &ENDIF
  END.
END.
&IF {&standalone} = NO &THEN
RUN DisplayProgress (cProgram, cAction, cTable , giTotal , giDone).
&ENDIF

/* aa-parameter -> default waarde in de database vullen, anders krijg ik dat verderop niet recht getrokken. */
FOR EACH aa-parameter WHERE aa-parameter.ind_year_prch = TRUE EXCLUSIVE-LOCK:
  IF aa-parameter.purch_inv_template = "" THEN aa-parameter.purch_inv_template = "yy999999".
END.